package dev.fitko.fitconnect.client.cmd;

import com.beust.jcommander.JCommander;
import com.beust.jcommander.ParameterException;
import dev.fitko.fitconnect.client.SenderClient;
import dev.fitko.fitconnect.client.SubscriberClient;
import dev.fitko.fitconnect.client.cmd.batch.CsvImporter;
import dev.fitko.fitconnect.client.cmd.commands.GetAllSubmissionsCommand;
import dev.fitko.fitconnect.client.cmd.commands.GetOneSubmissionCommand;
import dev.fitko.fitconnect.client.cmd.commands.ListAllSubmissionsCommand;
import dev.fitko.fitconnect.client.cmd.commands.SendBatchCommand;
import dev.fitko.fitconnect.client.cmd.commands.SendSubmissionCommand;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;

public class CommandLineClient {

    private static final Logger LOGGER = LoggerFactory.getLogger(CommandLineClient.class);

    private final SendSubmissionCommand sendSubmissionCommand;
    private final ListAllSubmissionsCommand listAllSubmissionsCommand;
    private final GetOneSubmissionCommand getOneSubmissionCommand;
    private final GetAllSubmissionsCommand getAllSubmissionsCommand;
    private final SendBatchCommand sendBatchCommand;

    private final CommandExecutor commandExecutor;

    private final JCommander jc;

    public CommandLineClient(final SenderClient.Start senderClient, final SubscriberClient.RequestSubmission subscriberClient) {

        this.sendSubmissionCommand = new SendSubmissionCommand();
        this.listAllSubmissionsCommand = new ListAllSubmissionsCommand();
        this.getOneSubmissionCommand = new GetOneSubmissionCommand();
        this.getAllSubmissionsCommand = new GetAllSubmissionsCommand();
        this.sendBatchCommand = new SendBatchCommand();

        this.commandExecutor = new CommandExecutor(senderClient, subscriberClient, new CsvImporter());

        this.jc = getJCommander();
    }

    public void run(final String... args) {
        final var parsedCommand = parseCommandFromArguments(args);
        if (parsedCommand == null) {
            jc.usage();
            return;
        }
        try {
            executeCommand(parsedCommand);
        } catch (final Exception e) {
            LOGGER.error(e.getMessage());
        }
    }

    private JCommander getJCommander() {
        return JCommander.newBuilder()
                .addCommand(sendSubmissionCommand)
                .addCommand(listAllSubmissionsCommand)
                .addCommand(getOneSubmissionCommand)
                .addCommand(getAllSubmissionsCommand)
                .addCommand(sendBatchCommand)
                .build();
    }

    private void executeCommand(final String parsedCommand) throws IOException {
        switch (parsedCommand) {
            case SendSubmissionCommand.SEND_CMD_NAME:
                commandExecutor.sendSubmission(sendSubmissionCommand);
                break;
            case ListAllSubmissionsCommand.LIST_CMD_NAME:
                commandExecutor.listSubmissions(listAllSubmissionsCommand);
                break;
            case GetOneSubmissionCommand.GET_CMD_NAME:
                commandExecutor.getOneSubmission(getOneSubmissionCommand);
                break;
            case GetAllSubmissionsCommand.GET_ALL_CMD_NAME:
                commandExecutor.getAllSubmissions(getAllSubmissionsCommand);
                break;
            case SendBatchCommand.SEND_BATCH_CMD_NAME:
                commandExecutor.sendBatch(sendBatchCommand);
                break;
            default:
                break;
        }
    }

    private String parseCommandFromArguments(final String[] args) {
        try {
            jc.parse(args);
            return jc.getParsedCommand();
        } catch (final ParameterException e) {
            LOGGER.error(e.getMessage(), "\n");
            return null;
        }
    }


}
